﻿#include "Obelisk.h"

#include "CGameController.h"
#include "CGameState.h"
#include "CCombat.h"
#include "CIsland.h"
#include "CBullet.h"
#include "CTurret.h"
#include "CRocket.h"

using namespace Obelisk;



CCombat::CCombat(IObelisk* aObelisk, CGameController* aOwner, CIsland* aIsland)
{
	mObelisk = aObelisk;
	mOwner = aOwner;
	mIsland = aIsland;

	mNumHelis = aOwner->Game()->GetNumHelis();

	const float Zn = 2, Zf = 1024;
	float a = Zf / (Zn - Zf);
	memset(&mProjectionMatrix, 0, 16 * sizeof(float));
	mProjectionMatrix[0 * 4 + 0] = 2 / (1024.0f / 100);
	mProjectionMatrix[1 * 4 + 1] = -2 * (1 / 0.75f) / (1024.0f / 100);
	mProjectionMatrix[2 * 4 + 2] = a;
	mProjectionMatrix[3 * 4 + 2] = -a * Zn;
	mProjectionMatrix[2 * 4 + 3] = 1;

	mObelisk->SetBackgroundColour(0.6862f, 0.9412f, 1.0f);

	mForTheWin = 0;

	for (int i = 0; i < aIsland->GetLevel() * 4 + 2; i++)
	{
		mTurrets.Add(new CTurret(mObelisk->Random(1600, 6000), aIsland->GetLevel()));
	}

	ResetHeli();
}



CCombat::~CCombat()
{
	foreach (mBullets)
		delete mBullets.Current();
	foreach (mTurrets)
		delete mTurrets.Current();
	foreach (mRockets)
		delete mRockets.Current();
}



bool CCombat::Update()
{
	if (mForTheWin != 0)
	{
		if (--mForTheWin == 0)
		{
			mOwner->Game()->WinIsland(mIsland);
			mOwner->FinishCombat();
			return true;
		}
		return false;
	}

	if (mDeathCountdown == -1)
	{
		bool mGoingDown = false;
		if (mObelisk->IsKeyDown(kUp))
		{
			if (mHeliY > 170)
			{
				mHeliY -= 5;
				mHeliState = hsFlying;
			}
		}
		else if (mObelisk->IsKeyDown(kDown))
		{
			if (mHeliState == hsFlying && mHeliY < 500)
			{
				mHeliY += 5;
				mGoingDown = true;
			}
		}
		if (mObelisk->IsKeyDown(kLeft))
		{
			if (mHeliState == hsFlying)
			{
				mHeliX -= 5;
				mHeliDir = 0;
			}
		}
		else if (mObelisk->IsKeyDown(kRight))
		{
			if (mHeliState == hsFlying)
			{
				mHeliX += 5;
				mHeliDir = 1;
			}
		}
		if (mObelisk->IsKeyDown(kSpace))
		{
			if (mBulletCountdown == 0)
			{
				mBulletCountdown = 20;
				mBullets.Add(new CBullet(mHeliX + (mHeliDir == 0 ? -30.0f : 30.0f), mHeliY,
						mHeliDir == 0 ? -20.0f : 20.0f, 3));
			}
		}

		if (mHeliX >= -55 && mHeliX <= 55)   // land on boat
		{
			if (mGoingDown && mHeliY >= 464)
			{
				mHeliY = 464;
				mHeliState = hsOnBoat;
			}
		}
		else if (mHeliX < -500)   // going too far left
			Die();
		else if (mHeliX > 7000)   // going too far right
			Die();
		else if (mHeliX < 1500)   // hit water
		{
			if (mHeliY >= 500)
				Die();
		}
		else if (mHeliY >= 500)
			mHeliState = hsOnLand;

		foreach (mBullets)
		{
			mBullets->Update();
			if (mBullets->GetY() > 500)
			{
				foreach (mTurrets)
				{
					if (!mTurrets->IsExploded() && mBullets->GetX() > mTurrets->GetX() - 42 &&
							mBullets->GetX() < mTurrets->GetX() + 42)
					{
						mTurrets->Explode();
						break;   // only explode one turret
					}
				}
				delete mBullets.Current();
				mBullets.Delete();
			}
		}
		if (mBulletCountdown > 0)
			mBulletCountdown--;
		foreach (mTurrets)
		{
			if (mTurrets->Update())
			{
				delete mTurrets.Current();
				mTurrets.Delete();
			}
			else if (!mTurrets->IsExploded() && mTurrets->CanShoot() && fabs(mHeliX -
					mTurrets->GetX()) < 600)
			{
				mTurrets->Shoot();
				float Angle = (float)(atan2(mHeliY - 450, mHeliX - (float)mTurrets->GetX()) *
						180 / M_PI) + (float)mObelisk->Random() * 20 - 10;
				mRockets.Add(new CRocket((float)mTurrets->GetX(), 450, Angle));
			}
		}
		if (mTurrets.GetCount() == 0)
		{
			mForTheWin = 2 * 50 + 25;
			mHeliFrame = 0;
			mHeliFrameCount = 0;
		}
		foreach (mRockets)
		{
			mRockets->Update();
			if (mRockets->GetX() > mHeliX - 50 && mRockets->GetX() < mHeliX + 50 &&
					mRockets->GetY() > mHeliY - 22 && mRockets->GetY() < mHeliY + 16)
			{
				delete mRockets.Current();
				mRockets.Delete();
				if (--mHP == 0)
				{
					Die();
					break;
				}
			}
		}

		mCamera = mHeliX;
		if (mCamera < 0)
			mCamera = 0;
		if (mCamera > 6500)
			mCamera = 6500;

		if (++mHeliFrameCount == 3)
		{
			mHeliFrameCount = 0;
			mHeliFrame = 1 - mHeliFrame;
		}
	}
	else
	{
		if (--mDeathCountdown == -1)
		{
			if (mNumHelis == 0)
			{
				mOwner->GameOver(SL("All Your Helicopters Were Destroyed"), 556);
				return true;
			}
			ResetHeli();
		}
	}
	return false;
}



void CCombat::Render()
{
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	glMultMatrixf(mProjectionMatrix);
	glTranslatef(-mCamera, 0, 0);
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();

	CVector WaterV0(-1500, 50, 200);
	CVector WaterV1(1500, 50, 200);
	CVector WaterV2(1500, 80, 10);
	CVector WaterV3(-1500, 80, 10);
	CColour WaterColour(0.2196f, 0.3098f, 1.0f);
	mObelisk->TriColour(WaterV0, WaterV1, WaterV2, WaterColour);
	mObelisk->TriColour(WaterV0, WaterV2, WaterV3, WaterColour);

	CVector SandV0(1500, 50, 200);
	CVector SandV1(4000, 50, 200);
	CVector SandV2(4000, 80, 10);
	CVector SandV3(1500, 80, 10);
	CColour SandColour(0.9373f, 0.7451f, 0.3843f);
	mObelisk->TriColour(SandV0, SandV1, SandV2, SandColour);
	mObelisk->TriColour(SandV0, SandV2, SandV3, SandColour);

	CVector GrassV0(4000, 50, 200);
	CVector GrassV1(8000, 50, 200);
	CVector GrassV2(8000, 80, 10);
	CVector GrassV3(4000, 80, 10);
	CColour GrassColour(0.5255f, 0.8824f, 0.1020f);
	mObelisk->TriColour(GrassV0, GrassV1, GrassV2, GrassColour);
	mObelisk->TriColour(GrassV0, GrassV2, GrassV3, GrassColour);

	glMatrixMode(GL_MODELVIEW);
	glTranslatef(0, -384, 0);
	glPushMatrix();
	glTranslatef(0, 474, 0);
	const float cBoatXSize = 70.0f;
	const float cBoatYSize = 58.0f;
	CVector BoatV0(-cBoatXSize, -cBoatYSize, 100);
	CVector BoatV1(cBoatXSize, -cBoatYSize, 100);
	CVector BoatV2(cBoatXSize, cBoatYSize, 100);
	CVector BoatV3(-cBoatXSize, cBoatYSize, 100);
	CTexture BoatTexture = mObelisk->GetTexture(SL("BoatRight.bmp"));
	mObelisk->TriTexture(BoatV0, 0.0, 0.0, BoatV1, 1.0, 0.0, BoatV2, 1.0, 1.0, BoatTexture);
	mObelisk->TriTexture(BoatV0, 0.0, 0.0, BoatV2, 1.0, 1.0, BoatV3, 0.0, 1.0, BoatTexture);
	glPopMatrix();

	foreach (mTurrets)
	{
		glPushMatrix();
		glTranslatef((float)mTurrets->GetX(), 500, 0);
		CVector TurretV0(-46.5, -77, 100);
		CVector TurretV1(46.5, -77, 100);
		CVector TurretV2(46.5, 0, 100);
		CVector TurretV3(-46.5, 0, 100);
		CTexture TurretTexture = mObelisk->GetTexture(mTurrets->GetX() > mHeliX ?
				SL("TurretLeft.bmp") : SL("TurretRight.bmp"));
		mObelisk->TriTexture(TurretV0, 0.0, 0.0, TurretV1, 1.0, 0.0, TurretV2, 1.0, 1.0, TurretTexture);
		mObelisk->TriTexture(TurretV0, 0.0, 0.0, TurretV2, 1.0, 1.0, TurretV3, 0.0, 1.0, TurretTexture);

		if (mTurrets->IsExploded())
		{
			CVector ExploV0(-67.5f, -120.0f, 100);
			CVector ExploV1(67.5f, -120.0f, 100);
			CVector ExploV2(67.5f, -6.0f, 100);
			CVector ExploV3(-67.5f, -6.0f, 100);
			CTexture ExploTexture = mObelisk->GetTexture(SL("Explosion.bmp"));
			mObelisk->TriTexture(ExploV0, 0.0, 0.0, ExploV1, 1.0, 0.0, ExploV2, 1.0, 1.0, ExploTexture);
			mObelisk->TriTexture(ExploV0, 0.0, 0.0, ExploV2, 1.0, 1.0, ExploV3, 0.0, 1.0, ExploTexture);
		}
		glPopMatrix();
	}

	glPushMatrix();
	glTranslatef(mHeliX, mHeliY, 0);
	const float cHeliXSize = 63.0f;
	const float cHeliYSize = 63.0f * 52 / 126;
	CVector HeliV0(-cHeliXSize, -cHeliYSize, 100);
	CVector HeliV1(cHeliXSize, -cHeliYSize, 100);
	CVector HeliV2(cHeliXSize, cHeliYSize, 100);
	CVector HeliV3(-cHeliXSize, cHeliYSize, 100);
	CTexture HeliTexture;
	if (mHeliDir == 0)
		HeliTexture = mObelisk->GetTexture(mHeliFrame == 0 ? SL("HeliLeft0.bmp"): SL("HeliLeft1.bmp"));
	else
		HeliTexture = mObelisk->GetTexture(mHeliFrame == 0 ? SL("HeliRight0.bmp"): SL("HeliRight1.bmp"));
	mObelisk->TriTexture(HeliV0, 0.0, 0.0, HeliV1, 1.0, 0.0, HeliV2, 1.0, 1.0, HeliTexture);
	mObelisk->TriTexture(HeliV0, 0.0, 0.0, HeliV2, 1.0, 1.0, HeliV3, 0.0, 1.0, HeliTexture);

	if (mDeathCountdown != -1)
	{
		CVector ExploV0(-67.5f, -69.0f, 100);
		CVector ExploV1(67.5f, -69.0f, 100);
		CVector ExploV2(67.5f, 45.0f, 100);
		CVector ExploV3(-67.5f, 45.0f, 100);
		CTexture ExploTexture = mObelisk->GetTexture(SL("Explosion.bmp"));
		mObelisk->TriTexture(ExploV0, 0.0, 0.0, ExploV1, 1.0, 0.0, ExploV2, 1.0, 1.0, ExploTexture);
		mObelisk->TriTexture(ExploV0, 0.0, 0.0, ExploV2, 1.0, 1.0, ExploV3, 0.0, 1.0, ExploTexture);
	}
	glPopMatrix();

	foreach (mRockets)
	{
		glPushMatrix();
		glTranslatef(mRockets->GetX(), mRockets->GetY(), 0);
		glRotatef(mRockets->GetAngle(), 0, 0, 1);
		CVector RocketV0(4, 0, 100);
		CVector RocketV1(-18, 4, 100);
		CVector RocketV2(-18, -4, 100);
		CColour RocketColour = mObelisk->Random(0, 1) == 0 ? CColour(1, 1, 1) : CColour(1, 1, 0);
		mObelisk->TriColour(RocketV0, RocketV1, RocketV2, RocketColour);
		glPopMatrix();
	}

	foreach (mBullets)
	{
		glPushMatrix();
		glTranslatef(mBullets->GetX(), mBullets->GetY(), 0);
		glRotatef((float)mObelisk->Random() * 60, 0, 0, 1);
		CVector BulletV0(0 * 10, -0.5f * 10, 100);
		CVector BulletV1(0.5f * 10, 0.5f * 10, 100);
		CVector BulletV2(-0.5f * 10, 0.5f * 10, 100);
		CColour BulletColour = mObelisk->Random(0, 1) == 0 ? CColour(1, 1, 1) : CColour(1, 1, 0);
		mObelisk->TriColour(BulletV0, BulletV1, BulletV2, BulletColour);
		glPopMatrix();
	}

	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();
	mObelisk->SetSpriteViewport(1024, 768);
	mObelisk->SetAlpha(0.6f);
	mOwner->DrawRect(0, 0, 260, 60, CColour(0.3f, 0.3f, 0.3f));
	mOwner->DrawRect(810, 0, 1023, 29, CColour(0.3f, 0.3f, 0.3f));
	if (mForTheWin != 0)
		mOwner->DrawRect(512 - 233, 384 - 40, 512 + 233, 384 - 10, CColour(0.3f, 0.3f, 0.3f));
	mObelisk->SetAlpha(1.0f);
	mObelisk->DrawText(SL("Helicopters: ") + mObelisk->IntToString(mNumHelis), 10, 0);
	mObelisk->DrawText(SL("HP: ") + mObelisk->IntToString(mHP), 10, 30);
	mObelisk->DrawText(SL("Turrets: ") + mObelisk->IntToString(mTurrets.GetCount()),
			830, 0);
	if (mForTheWin != 0)
		mObelisk->DrawText(SL("You have conquered the island!"), 512 - 233, 384 - 40);
}



void CCombat::Die()
{
	mOwner->Game()->SetNumHelis(mOwner->Game()->GetNumHelis() - 1);
	mNumHelis--;
	mDeathCountdown = 45;
	mHeliFrame = 0;
	mHeliFrameCount = 0;
}



void CCombat::ResetHeli()
{
	mCamera = 0;

	mDeathCountdown = -1;
	mHP = 4 + mOwner->Game()->GetRank() * 2;
	mHeliX = 0;
	mHeliY = 464;
	mHeliDir = 1;
	mHeliFrame = 0;
	mHeliFrameCount = 0;
	mHeliState = hsOnBoat;

	foreach (mBullets)
		delete mBullets.Current();
	mBullets = CCollection<CBullet>();
	mBulletCountdown = 0;
	foreach (mTurrets)
	{
		if (mTurrets->IsExploded())
		{
			delete mTurrets.Current();
			mTurrets.Delete();
		}
		else
			mTurrets->Reset();
	}
	foreach (mRockets)
		delete mRockets.Current();
	mRockets = CCollection<CRocket>();
}